﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using HeuristicLab.Clients.Hive.CloudManager.Model;

namespace HeuristicLab.Clients.Hive.CloudManager.Azure {
  public static class ServiceManagementOperation {

    #region Namespaces

    private static XNamespace wa = Constants.NSWindowsAzure;
    private static XNamespace sc = Constants.NSSchemaMicrosoft;
    private static XNamespace sh = Constants.NSServiceConfiguration;

    #endregion

    #region Public Methods

    public static Subscription GetSubscriptionInfo(string subscriptionId, string thumbprint) {
      string uri = string.Format(Constants.URISubscriptionFormat, subscriptionId);
      ServiceWebRequest operation = new ServiceWebRequest(thumbprint);
      XDocument response = operation.Invoke(uri);

      Subscription subscription = new Subscription();
      subscription.SubscriptionID = response.Descendants(wa + "SubscriptionID").Single().Value;
      subscription.SubscriptionName = response.Descendants(wa + "SubscriptionName").Single().Value;
      subscription.SubscriptionStatus = response.Descendants(wa + "SubscriptionStatus").Single().Value;
      subscription.MaxCoreCount = Convert.ToInt32(response.Descendants(wa + "MaxCoreCount").Single().Value);
      subscription.MaxHostedServices = Convert.ToInt32(response.Descendants(wa + "MaxHostedServices").Single().Value);
      subscription.MaxStorageAccounts = Convert.ToInt32(response.Descendants(wa + "MaxStorageAccounts").Single().Value);
      subscription.CurrentCoreCount = Convert.ToInt32(response.Descendants(wa + "CurrentCoreCount").Single().Value);
      subscription.CurrentHostedServices = Convert.ToInt32(response.Descendants(wa + "CurrentHostedServices").Single().Value);
      subscription.CurrentStorageAccounts = Convert.ToInt32(response.Descendants(wa + "CurrentStorageAccounts").Single().Value);
      subscription.CertificateThumbprint = thumbprint;

      return subscription;
    }

    public static List<string> ListLocations(string subscriptionId, string thumbprint) {
      string uri = string.Format(Constants.URILocationsFormat, subscriptionId);
      ServiceWebRequest operation = new ServiceWebRequest(thumbprint);
      XDocument response = operation.Invoke(uri);
      List<string> result = null;
      if (response.Root.HasElements) {
        result = (from s in response.Descendants(wa + "Name")
                  select s.Value).ToList();
      }
      return result;
    }

    #endregion

    #region Private Methods

    private static int GetInstanceCount(XElement configuration, string roleName) {
      XElement instanceElement = (from s in configuration.Elements(sh + "Role")
                                  where s.Attribute("name").Value == roleName
                                  select s.Element(sh + "Instances")).First();
      int instanceCount = Convert.ToInt32(instanceElement.Attribute("count").Value);
      return instanceCount;
    }

    private static void SetInstanceCount(XElement configuration, string roleName, int value) {
      XElement instanceElement = (from s in configuration.Elements(sh + "Role")
                                  where s.Attribute("name").Value == roleName
                                  select s.Element(sh + "Instances")).First();
      instanceElement.SetAttributeValue("count", value);
    }

    private static string CreateHostedService(string subscriptionId, string thumbprint, string serviceName, string label, string description, string location, string affinityGroup) {
      string uri = string.Format(Constants.URIHostedServiceFormat, subscriptionId);
      XDocument payload = CreateHostedServicePayload(serviceName, label, description, location, affinityGroup);
      ServiceWebRequest operation = new ServiceWebRequest(thumbprint);
      string requestId = operation.Invoke(uri, payload);
      return requestId;
    }

    private static XDocument CreateHostedServicePayload(string serviceName, string label, string description, string location, string affinityGroup) {
      string base64LabelName = Utils.ConvertToBase64String(label);
      XElement xServiceName = new XElement(wa + "ServiceName", serviceName);
      XElement xLabel = new XElement(wa + "Label", base64LabelName);
      XElement xDescription = new XElement(wa + "Description", description);
      XElement createHostedService = new XElement(wa + "CreateHostedService");
      createHostedService.Add(xServiceName);
      createHostedService.Add(xLabel);
      createHostedService.Add(xDescription);

      if (location != String.Empty) {
        XElement xLocation = new XElement(wa + "Location", location);
        createHostedService.Add(xLocation);
      } else {
        XElement xAffinityGroup = new XElement(wa + "AffinityGroup", affinityGroup);
        createHostedService.Add(xAffinityGroup);
      }

      XDocument payload = new XDocument();
      payload.Add(createHostedService);
      payload.Declaration = new XDeclaration("1.0", "UTF-8", "no");

      return payload;
    }

    private static XDocument CreateAddCertificatePayload(string certificateFilePath, string password) {
      XElement xData = new XElement(wa + "Data", Convert.ToBase64String(File.ReadAllBytes(certificateFilePath)));
      XElement xCertificateFormat = new XElement(wa + "CertificateFormat", "pfx");
      XElement xPassword = new XElement(wa + "Password", password);

      XElement addcert = new XElement(wa + "CertificateFile");
      addcert.Add(xData);
      addcert.Add(xCertificateFormat);
      addcert.Add(xPassword);

      XDocument payload = new XDocument();
      payload.Add(addcert);
      payload.Declaration = new XDeclaration("1.0", "UTF-8", "no");

      return payload;
    }

    private static XDocument CreateDeploymentPayload(string deploymentName, string packageUrl, string pathToConfigurationFile, string label) {
      string configurationFile = File.ReadAllText(pathToConfigurationFile);
      XElement xConfig = XElement.Parse(configurationFile);
      return CreateDeploymentPayload(deploymentName, packageUrl, xConfig, label);
    }

    private static XDocument CreateDeploymentPayload(string deploymentName, string packageUrl, XElement configuration, string label) {
      string base64ConfigurationFile = Utils.ConvertToBase64String(configuration.ToString());
      string base64Label = Utils.ConvertToBase64String(label);

      XElement xName = new XElement(wa + "Name", deploymentName);
      XElement xPackageUrl = new XElement(wa + "PackageUrl", packageUrl);
      XElement xLabel = new XElement(wa + "Label", base64Label);
      XElement xConfiguration = new XElement(wa + "Configuration", base64ConfigurationFile);
      XElement xStartDeployment = new XElement(wa + "StartDeployment", "true");
      XElement xTreatWarningsAsError = new XElement(wa + "TreatWarningsAsError", "false");

      XElement createDeployment = new XElement(wa + "CreateDeployment");
      createDeployment.Add(xName);
      createDeployment.Add(xPackageUrl);
      createDeployment.Add(xLabel);
      createDeployment.Add(xConfiguration);
      createDeployment.Add(xStartDeployment);
      createDeployment.Add(xTreatWarningsAsError);

      XDocument payload = new XDocument();
      payload.Add(createDeployment);
      payload.Declaration =
      new XDeclaration("1.0", "UTF-8", "no");
      return payload;
    }

    private static XElement LoadConfiguration(String subscriptionId, String thumbprint, String serviceName, String deploymentSlot) {
      string uri = string.Format(Constants.URIDeploymentFormat, subscriptionId, serviceName, deploymentSlot);
      ServiceWebRequest operation = new ServiceWebRequest(thumbprint);
      XDocument deployment = operation.Invoke(uri);
      string base64Configuration = deployment.Element(wa + "Deployment").Element(wa + "Configuration").Value;
      string stringConfiguration = Utils.ConvertFromBase64String(base64Configuration);
      XElement configuration = XElement.Parse(stringConfiguration);
      return configuration;
    }

    private static string SaveConfiguration(String subscriptionId, String thumbprint, String serviceName, String deploymentSlot, XElement configuration) {
      string uri = string.Format(Constants.URIDeploymentConfigurationFormat, subscriptionId, serviceName, deploymentSlot);
      XDocument payload = CreateConfigurationPayload(configuration);
      ServiceWebRequest operation = new ServiceWebRequest(thumbprint);
      string requestId = operation.Invoke(uri, payload);
      return requestId;
    }

    private static XDocument CreateConfigurationPayload(XElement configuration) {
      string configurationString = configuration.ToString();
      string base64Configuration = Utils.ConvertToBase64String(configurationString);

      XElement xConfiguration = new XElement(wa + "Configuration", base64Configuration);
      XElement xChangeConfiguration = new XElement(wa + "ChangeConfiguration", xConfiguration);

      XDocument payload = new XDocument();
      payload.Add(xChangeConfiguration);
      payload.Declaration = new XDeclaration("1.0", "UTF-8", "no");

      return payload;
    }

    private static XDocument GetConfigurationFromDeployment(string subscriptionId, string thumbprint, string serviceName, string deploymentName) {
      //Deployment deployment = GetDeployment(subscriptionId, thumbprint, serviceName, deploymentName);
      //XDocument config = XDocument.Parse(deployment.Configuration);
      return null;//config;
    }

    private static bool ContainsRoleNameInConfiguration(XDocument configuration, string roleName) {
      if (configuration.Root.Name != sh + "ServiceConfiguration")
        throw new ArgumentException("No valid configuration", "configuration");


      IEnumerable<XElement> role = (from s in configuration.Root.Elements(sh + "Role")
                                    where s.Attribute("name").Value == roleName
                                    select s);
      return role.Count() > 0;
    }

    #endregion
  }
}
